import logging
logger = logging.getLogger('upkg')
# mailogger = logging.getLogger('upkg-maillogger')

import sys
# import smtplib
# from tornado.platform.epoll import EPollIOLoop
from tornado.ioloop import IOLoop


# class UpkgIOLoop(EPollIOLoop, object):
class UpkgIOLoop(IOLoop):
    """UpkgIOLoop
    """

   #  def initialize(self, **kwargs):
   #      """todo: Docstring for initialize

   #      :param **kwargs: arg description
   #      :type **kwargs: type description
   #      :return:
   #      :rtype:
   #      """
   #      super(UpkgIOLoop, self).initialize(**kwargs)
   # #initialize()

    def handle_callback_exception(self, callback):
        # We setup a Pokemon handler
        # to e-mail us when something
        # funky happens

        import traceback
        import socket
        import platform

        t, v, tr = sys.exc_info()
        estr = "IOLoop Excption\n"
        estr += "hostname: %s\n" % socket.gethostname()
        estr += "uname: %s\n" % ' '.join(platform.uname())
        estr += "Type: %s\n" % t
        estr += "Value: %s\n" % v
        estr += "Traceback: %s\n" % traceback.print_tb(tr)

        logger.error(estr, exc_info=True)
        # try:
        #     mailogger.exception(estr)
        # except smtplib.SMTPServerDisconnected:
        #     logger.error("Unable to mail exception")

        # for srv in self._services:
        #     srv.reset_timeout()
        # # end for srv in self._services

        # sys.exit(1)

        super(UpkgIOLoop, self).handle_callback_exception(callback)
    #handle_callback_exception()

    # def start(self):
    #     super(UpkgIOLoop, self).start()
    # #start()
#UpkgIOLoop



import logging
logger = logging.getLogger('upkg')

# import dateutil
import dateutil.parser
import dateutil.relativedelta
import datetime
import json
import tornado.httpclient
# from tornado import gen


class Search(object):
    """Docstring for Search """

    def __init__(self, async=False):
        """todo: to be defined """

        self._headers = {
            'User-Agent': "https://github.com/jeffbuttars/upkg",
            'Accept': "application/vnd.github.preview",  # Acknowledge the dev API
        }

        # A list of prefered repos to give greater weight in a search
        # and are searched first.
        self._prefered = None

        self._hc = tornado.httpclient.HTTPClient()
        # # self._ahc = tornado.httpclient.AsyncHTTPClient()

        self._gh_url_base = "https://api.github.com/search/repositories?per_page=100&q="
        self._last_result = None
    #__init__()

    def __call__(self, term, cb=None):
        """todo: Docstring for __call__

        :param term: arg description
        :type term: type description
        :return:
        :rtype:
        """

        logger.debug("term: %s, cb: %s", term, cb)

        if not term:
            return term

        url = self._gh_url_base + str(term)
        future = self._hc.fetch(url, method="GET", headers=self._headers)

        if future.code != 200:
            estr = "Error while search for {}\nerror code: {}, {}\n{}".format(
                term, future.code, future.error, future.body
            )
            raise Exception(estr)

        self._last_result = json.loads(future.body)
        return self._last_result
    #__call__()

    def __str__(self, res=None):
        """todo: Docstring for __str__

        :param res: arg description
        :type res: type description
        :return:
        :rtype:
        """

        res = res or self._last_result
        items = res['items']
        total_count = res['total_count']

        summary = "Showing {} of {} found items:\n".format(len(items), total_count)
        out = summary

        i = 1
        for item in items:
            # 2013-10-10T10:26:32Z
            # lu = datetime.datetime.strptime(item['updated_at'], '%Y-%m-%dT%H:%M:%SZ')
            # lu = datetime.datetime.strptime(item['updated_at'], '%Y-%m-%dT%H:%M:%SZ')

            # Strip the timezone, treat all time as UTC
            # td = self._fmt_tdelta(datetime.datetime.now() - lu)
            td = self._fmt_tdelta(dateutil.parser.parse(item['updated_at'][:-1]))

            fargs = {
                'url': item['url'].replace(
                    'https://api.github.com/repos/', '', 1
                ),
                'private': '\033[92mPublic\033[0m',
                'updated': '\033[93m' + td + '\033[0m',
                'description': item['description'].encode('ascii', 'ignore'),
            }

            if not fargs['private']:
                fargs['private'] = '\033[91mPrivate\033[0m',

            out += ('\033[94m' + str(i) + '\033[0m {url}\n\t'
                    "{description}\n\t"
                    "Last Update {updated}\n\t"
                    "Access: {private}\n\n").format(**fargs)
            i += 1
        # end for item in items

        out += summary
        return out
    #__str__()

    def _fmt_tdelta(self, td):
        """todo: Docstring for _fmt_tdelta

        :param td: arg description
        :type td: type description
        :return:
        :rtype:
        """

        rel = dateutil.relativedelta.relativedelta(datetime.datetime.utcnow(), td)

        if rel.years:
            return "{} Years and {} Months Ago".format(rel.years, rel.months)
        if rel.months:
            return "{} Months Ago".format(rel.months)
        if rel.days:
            return "{} Days Ago".format(rel.days)
        if rel.hours:
            return "{} Hours Ago".format(rel.hours)

        return "{} Minutes Ago".format(rel.minutes)
    #_fmt_tdelta()
#Search

"""
 {
      "archive_url": "https://api.github.com/repos/benfoster/Fabrik.Common/{archive_format}{/ref}", 
      "assignees_url": "https://api.github.com/repos/benfoster/Fabrik.Common/assignees{/user}", 
      "blobs_url": "https://api.github.com/repos/benfoster/Fabrik.Common/git/blobs{/sha}", 
      "branches_url": "https://api.github.com/repos/benfoster/Fabrik.Common/branches{/branch}", 
      "clone_url": "https://github.com/benfoster/Fabrik.Common.git", 
      "collaborators_url": "https://api.github.com/repos/benfoster/Fabrik.Common/collaborators{/collaborator}", 
      "comments_url": "https://api.github.com/repos/benfoster/Fabrik.Common/comments{/number}", 
      "commits_url": "https://api.github.com/repos/benfoster/Fabrik.Common/commits{/sha}", 
      "compare_url": "https://api.github.com/repos/benfoster/Fabrik.Common/compare/{base}...{head}", 
      "contents_url": "https://api.github.com/repos/benfoster/Fabrik.Common/contents/{+path}", 
      "contributors_url": "https://api.github.com/repos/benfoster/Fabrik.Common/contributors", 
      "created_at": "2012-07-05T13:34:05Z", 
      "default_branch": "master", 
      "description": "Useful stuff from fabrik", 
      "downloads_url": "https://api.github.com/repos/benfoster/Fabrik.Common/downloads", 
      "events_url": "https://api.github.com/repos/benfoster/Fabrik.Common/events", 
      "fork": false, 
      "forks": 23, 
      "forks_count": 23, 
      "forks_url": "https://api.github.com/repos/benfoster/Fabrik.Common/forks", 
      "full_name": "benfoster/Fabrik.Common", 
      "git_commits_url": "https://api.github.com/repos/benfoster/Fabrik.Common/git/commits{/sha}", 
      "git_refs_url": "https://api.github.com/repos/benfoster/Fabrik.Common/git/refs{/sha}", 
      "git_tags_url": "https://api.github.com/repos/benfoster/Fabrik.Common/git/tags{/sha}", 
      "git_url": "git://github.com/benfoster/Fabrik.Common.git", 
      "has_downloads": true, 
      "has_issues": true, 
      "has_wiki": true, 
      "homepage": null, 
      "hooks_url": "https://api.github.com/repos/benfoster/Fabrik.Common/hooks", 
      "html_url": "https://github.com/benfoster/Fabrik.Common", 
      "id": 4906940, 
      "issue_comment_url": "https://api.github.com/repos/benfoster/Fabrik.Common/issues/comments/{number}", 
      "issue_events_url": "https://api.github.com/repos/benfoster/Fabrik.Common/issues/events{/number}", 
      "issues_url": "https://api.github.com/repos/benfoster/Fabrik.Common/issues{/number}", 
      "keys_url": "https://api.github.com/repos/benfoster/Fabrik.Common/keys{/key_id}", 
      "labels_url": "https://api.github.com/repos/benfoster/Fabrik.Common/labels{/name}", 
      "language": "C#", 
      "languages_url": "https://api.github.com/repos/benfoster/Fabrik.Common/languages", 
      "master_branch": "master", 
      "merges_url": "https://api.github.com/repos/benfoster/Fabrik.Common/merges", 
      "milestones_url": "https://api.github.com/repos/benfoster/Fabrik.Common/milestones{/number}", 
      "mirror_url": null, 
      "name": "Fabrik.Common", 
      "notifications_url": "https://api.github.com/repos/benfoster/Fabrik.Common/notifications{?since,all,participating}", 
      "open_issues": 9, 
      "open_issues_count": 9, 
      "owner": {
        "avatar_url": "https://2.gravatar.com/avatar/3192eeb39449a86afa994fa9efc83179?d=https%3A%2F%2Fidenticons.github.com%2F061cdd38a3f82458730d861a54852ec7.png", 
        "events_url": "https://api.github.com/users/benfoster/events{/privacy}", 
        "followers_url": "https://api.github.com/users/benfoster/followers", 
        "following_url": "https://api.github.com/users/benfoster/following{/other_user}", 
        "gists_url": "https://api.github.com/users/benfoster/gists{/gist_id}", 
        "gravatar_id": "3192eeb39449a86afa994fa9efc83179", 
        "html_url": "https://github.com/benfoster", 
        "id": 827305, 
        "login": "benfoster", 
        "organizations_url": "https://api.github.com/users/benfoster/orgs", 
        "received_events_url": "https://api.github.com/users/benfoster/received_events", 
        "repos_url": "https://api.github.com/users/benfoster/repos", 
        "site_admin": false, 
        "starred_url": "https://api.github.com/users/benfoster/starred{/owner}{/repo}", 
        "subscriptions_url": "https://api.github.com/users/benfoster/subscriptions", 
        "type": "User", 
        "url": "https://api.github.com/users/benfoster"
      }, 
      "private": false, 
      "pulls_url": "https://api.github.com/repos/benfoster/Fabrik.Common/pulls{/number}", 
      "pushed_at": "2013-07-25T15:36:55Z", 
      "score": 13.695881, 
      "size": 4625, 
      "ssh_url": "git@github.com:benfoster/Fabrik.Common.git", 
      "stargazers_url": "https://api.github.com/repos/benfoster/Fabrik.Common/stargazers", 
      "statuses_url": "https://api.github.com/repos/benfoster/Fabrik.Common/statuses/{sha}", 
      "subscribers_url": "https://api.github.com/repos/benfoster/Fabrik.Common/subscribers", 
      "subscription_url": "https://api.github.com/repos/benfoster/Fabrik.Common/subscription", 
      "svn_url": "https://github.com/benfoster/Fabrik.Common", 
      "tags_url": "https://api.github.com/repos/benfoster/Fabrik.Common/tags", 
      "teams_url": "https://api.github.com/repos/benfoster/Fabrik.Common/teams", 
      "trees_url": "https://api.github.com/repos/benfoster/Fabrik.Common/git/trees{/sha}", 
      "updated_at": "2013-10-15T04:02:40Z", 
      "url": "https://api.github.com/repos/benfoster/Fabrik.Common", 
      "watchers": 57, 
      "watchers_count": 57
    }
"""



import logging
logger = logging.getLogger('upkg')

from upkg.cmds.base import BaseCmd
from upkg.lib import Repo


class Cmd(BaseCmd):
    """Docstring for Search """

    name = 'update'
    help_text = ("update all or some upkg")
    aliases = ['up', 'upd', 'upda', 'updat']

    def build(self):
        """todo: Docstring for build
        :return:
        :rtype:
        """

        self._cmd_parser.add_argument(
            'update',
            type=str,
            default=None,
            nargs="*",
            help=(""),
        )

        return super(Cmd, self).build()
    #build()

    def exec(self, args):
        """todo: Docstring for exec

        :param args: arg description
        :type args: type description
        :return:
        :rtype:
        """

        logger.debug("update %s", args.update)
        self.update(args.update) 
    #exec()

    def update(self, repos):
        """

        :param repos: arg description
        :type repos: type description
        :return:
        :rtype:
        """
        logger.debug("repos: %s", repos)

        if repos:
            rlist = [Repo(name=x) for x in repos]
        else:
            # Update them all!
            rlist = Repo.installed_list()

        logger.debug("repo list: %s", rlist)

        for r in rlist:
            logger.debug("calling update on: %s", r)
            r.update()
        # end for r in rlist
    #update()
# Cmd



import os
import inspect
from argparse import Namespace
from importlib.machinery import SourceFileLoader


def _clean_path(p):
    """todo: Docstring for _clen_path

    :param p: arg description
    :type p: type description
    :return:
    :rtype:
    """

    np = os.path.expanduser(p)
    np = os.path.abspath(np)

    return np
#_clean_path()


# Set config defaults
_config = {
    # Default path to config file.
    "cfg_path": _clean_path("~/.config/upkg/upkg_cfg.py"),

    # The install destination directory
    'upkg_destdir': _clean_path('~/.upkg'),

    # Require the user to have root access to use upkg
    'require_root': False,
}


settings = Namespace(**_config)


def load_settings(cfg_path=None):
    """todo: Docstring for load_settings

    :param cfg_path: arg description
    :type cfg_path: type description
    :return:
    :rtype:
    """

    global settings

    cfg_path = cfg_path or _config['cfg_path']

    cfg_d = _config.copy()

    if os.path.exists(cfg_path):
        sfl = SourceFileLoader('upkg_cfg', cfg_path)
        cfg_mod = sfl.load_module()

        for m in inspect.getmembers(cfg_mod):
            if m[0][0] != '_':
                cfg_d[m[0]] = m[1]
        # end for m in inspect.getme

    # Make the paths absolute.
    cfg_d["cfg_path"] = _clean_path(cfg_d["cfg_path"])
    cfg_d["upkg_destdir"] = _clean_path(cfg_d["upkg_destdir"])

    settings = Namespace(**cfg_d)
#load_settings()



def build(sub_parser, cmds):
    """todo: Docstring for build

    :param sub_parser: arg description
    :type sub_parser: type description
    :return:
    :rtype:
    """

    res = {}
    for cmd in cmds:
        res[cmd.name] = cmd(sub_parser)
    # end for cmd in cmds

    return res
#build()



import logging
logger = logging.getLogger('upkg')

import os

from upkg.cmds.base import BaseCmd
from upkg.conf import settings
from upkg.lib import Repo


class Cmd(BaseCmd):
    """Docstring for Search """

    name = 'install'
    help_text = ("install upkg")
    aliases = ['in', 'ins', 'inst', 'insta', 'instal']

    def build(self):
        """todo: Docstring for build
        :return:
        :rtype:
        """

        self._cmd_parser.add_argument(
            'install',
            type=str,
            default=None,
            nargs="+",
            help=(""),
        )

        self._cmd_parser.add_argument(
            '-l',
            '--location',
            default=None,
            help=("Specify the installation location. ")
        )

        return super(Cmd, self).build()

    def install_repo(self, repo, location=''):
        """todo: Docstring for install_repo
        :param repo: arg description
        :type repo: type description
        :return:
        :rtype:
        """

        dst = location or settings.upkg_destdir

        # make sure the destination dir exists.
        if not os.path.exists(dst):
            os.makedirs(dst)

        # repo = Repo(url=repo, repo_dir=dst)
        repo = Repo(url=repo)
        repo.install()

        return repo

    def exec(self, args):
        """todo: Docstring for exec

        :param args: arg description
        :type args: type description
        :return:
        :rtype:
        """

        logger.debug("install %s, location %s", args.install, args.location)
        location = args.location and os.path.abspath(args.location)

        if location and len(args) > 1:
            raise Exception(("You cannot specify multiple install packages when "
                             "using the --location option."
                            ))

        for repo in args.install:
            self.install_repo(repo, location)



import logging
logger = logging.getLogger('upkg')

_CTX = None


class UpkgCtx(object):

    def __init__(self):
        """todo: to be defined """
        self._deps = {}
        self._installed = {}
    #__init__()

    def add_dep(self, name, dep):
        """todo: Docstring for add_dep

        :param name: arg description
        :type name: type description
        :param dep: arg description
        :type dep: type description
        :return:
        :rtype:
        """
        logger.debug("name: %s, dep: %s", name, dep)

        if dep not in self._deps:
            self._deps[name] = dep
    #add_dep()

    @property
    def deps(self):
        return self._deps

    @property
    def deps_needed(self):
        needed = []
        for k in self._deps:
            try:
                if k not in self._installed:
                    needed.append(self._deps[k])
            except KeyError:
                pass
        return needed

    def dep(self, name):
        return self._deps[name]

    def installed(self, name):
        """todo: Docstring for installed

        :param name: arg description
        :type name: type description
        :return:
        :rtype:
        """
        logger.debug("name: %s", name)

        self._installed[name] = True
        return name
    #installed()
#UpkgCtx


def get_ctx():
    """todo: Docstring for get_ctx
    :return:
    :rtype:
    """
    return _CTX or UpkgCtx()
#get_ctx()



import logging
logger = logging.getLogger('upkg')


from upkg.cmds.base import BaseCmd
from upkg.lib import Repo


class Cmd(BaseCmd):
    """Docstring for Search """

    name = 'remove'
    help_text = ("remove installed pkg(s)")

    def build(self):
        """todo: Docstring for build
        :return:
        :rtype:
        """

        self._cmd_parser.add_argument(
            'remove',
            type=str,
            default=None,
            nargs="+",
            help=("Remove an installed pkg"),
        )

        return super(Cmd, self).build()
    #build()

    def exec(self, args):
        """todo: Docstring for exec

        :param args: arg description
        :type args: type description
        :return:
        :rtype:
        """

        logger.debug("remove %s", args.remove)

        for pkg in args.remove:
            logger.debug("removing %s", pkg)
            self.remove_repo(pkg)
        # end for repo in args.remove
    #exec()

    def remove_repo(self, rname):
        """todo: Docstring for remove_repo

        :param repo: arg description
        :type repo: type description
        :return:
        :rtype:
        """
        logger.debug("%s", rname)
        repo = Repo(name=rname)
        repo.remove()

        return repo
    #remove_repo()
# Cmd



from upkg.cmds.base import BaseCmd


class Cmd(BaseCmd):
    """Docstring for Search """

    name = 'search'
    help_text = ("search for upkg")

    def build(self):
        """todo: Docstring for build
        :return:
        :rtype:
        """

        # self._cmd_parser.add_argument(
        #     'search',
        #     type=str,
        #     default=None,
        #     # nargs="?",
        #     nargs=1,
        #     help=("Search for a package/repo by name"),
        # )

        return super(Cmd, self).build()
    #build()
# Cmd





class Install(object):
    """Docstring for Install """

    # def __init__(self):
    #     """todo: to be defined """
    #     pass    
    # #__init__()

    def __call__(self, *args):
        """todo: Docstring for __call__

        :param *args: arg description
        :type *args: type description
        :return:
        :rtype:
        """

        for pkg in args:
            self.install(pkg)
        # end for pkg in args
    #__call__()

    def install(self, pkg_url):
        """todo: Docstring for install

        :param pkg_url: arg description
        :type pkg_url: type description
        :return:
        :rtype:
        """

        pass
    #install()
#Install



import sys
import os
import importlib
import glob


# Imoprt and instantiate each Cmd object.
_this_dir = os.path.dirname(__file__)
_this_mod = os.path.basename(_this_dir)


def build_cmds(sub_parser):

    cmd_objs = {}
    imlist = glob.glob(os.path.join(_this_dir, "*.py"))
    imlist.remove(os.path.join(_this_dir, "__init__.py"))
    imlist.remove(os.path.join(_this_dir, "base.py"))

    imlist = [os.path.basename(x) for x in imlist]
    imlist = [os.path.splitext(x)[0] for x in imlist]

    for im in imlist:
        mod = importlib.import_module(_this_mod + '.' + im)
        if hasattr(mod, 'Cmd'):
            cmd_objs[mod.Cmd.name] = mod.Cmd(sub_parser)
            cmd_objs[mod.Cmd.name].build()

    return cmd_objs



import logging
logger = logging.getLogger('upkg')

import os
import sys
import subprocess

try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO

from blessings import Terminal

import shutil
from urllib.parse import urlparse
from pprint import pformat as pf
import sh
from sh import git

from conf import settings
from . import ctx as upkg_ctx

sh.logging_enabled = True

ugly_ext = ('.git', '.hg')


class InvalidRepo(Exception):
    """Docstring for InvalidRepo """
    pass
#InvalidRepo


def repo_dirlist():
    """List out the directories in the install location(s)
    :return:
    :rtype:
    """

    for d in os.listdir(settings.upkg_destdir):
        dp = os.path.join(settings.upkg_destdir, d)
        # logger.debug("%s", dp)
        if os.path.isdir(dp) and d[0] != '.':
            # logger.debug("adding %s", dp)
            yield {'base': d, 'root': settings.upkg_destdir,
                   'path': dp}
#repo_dirlist()


def did_u_mean(name):
    """todo: Docstring for did_u_mean

    :param name: arg description
    :type name: type description
    :return:
    :rtype:
    """
    logger.debug("%s", name)

    dlist = Repo.installed_list()
    res = []
    for d in dlist:
        if name in d.name or d.name in name:
            res.append(nice_pkg_name(d.name))
    # end for d in dlist

    if res:
        return "Did you mean '" + " or ".join(res) + "'?"

    return ""
#did_u_mean()


def pkg_name_to_path(pkgname):
    """todo: Docstring for pkg_name_to_path

    :param pkgname: arg description
    :type pkgname: type description
    :return:
    :rtype:
    """

    logger.debug("'%s'", pkgname)

    fp = os.path.join(settings.upkg_destdir, pkgname)
    if os.path.isdir(fp):
        logger.debug("found %s", fp)
        return fp

    # Try to find the repo dir if just needs an extension
    for d in repo_dirlist():
        logger.debug("trying %s", d)
        if d['base'].startswith(pkgname):
            # logger.debug("startwith")
            root, ext = os.path.splitext(d['base'])
            if pkgname == root:
                logger.debug("found %s", d)
                return d['path']
    # end for d in dlist

    logger.debug("found nothing")
    return ""
#pkg_name_to_path()


def nice_pkg_name(name):
    """todo: Docstring for nice_pkg_name

    :param name: arg description
    :type name: type description
    :return:
    :rtype:
    """
    logger.debug("%s", name)

    root, ext = os.path.splitext(name)
    logger.debug("root :'%s', ext: '%s'", root, ext)

    if ext in ugly_ext:
        logger.debug("remove ext %s to get %s", ext, root)
        return root

    logger.debug("no change %s", name)
    return name
#nice_pkg_name()


class Repo(object):
    """
        Repo object. Perform git operations on a given repo.
    """

    def __init__(self, repo_dir="", name="", url=""):
        """todo: to be defined

        :param repo_dir: arg description
        :type repo_dir: type description
        :param name: arg description
        :type name: type description
        :param url: arg description
        :type url: type description
        """
        logger.debug("repo_dir: %s, name: %s, url: %s", repo_dir, name, url)

        if not (repo_dir or name or url):
            estr = "You must supply at least one of: repo_dir, name, or url."
            logger.error(estr)
            raise InvalidRepo(estr)

        self._repo_dir = repo_dir
        self._name = nice_pkg_name(name)
        self._url = url
        self._basename = ""

        self.supported_schemes = ('git', 'https', 'http', 'file', '')


        self._ctx = upkg_ctx.get_ctx()
        self.term = Terminal()
    #__init__()

    def __repr__(self):
        """todo: Docstring for __repr__
        :return:
        :rtype:
        """

        return "{} URL:{} LOCATION: {}".format(
            self._name,
            self._url,
            self._repo_dir
        )
    #__repr__()

    def _build_writer(self, output, color=None, prefix=""):
        if color:

            def _cwriter(line):
                go = getattr(self.term, color)
                output(go(self.name + ": " + prefix + line))

            return _cwriter

        def _writer(line):
            output(self.name + ": " + prefix + line)

        return _writer
    #_build_writer()

    def _sh_stdout(self, *args, **kwargs):
        return self._build_writer(sys.stdout.write, *args, **kwargs)
    #_sh_stdout()

    def _sh_stderr(self, *args, **kwargs):
        return self._build_writer(sys.stderr.write, *args, **kwargs)
    #_sh_stderr()

    def __str__(self):
        """todo: Docstring for __str__
        :return:
        :rtype:
        """

        return "{} : {}".format(self.name, self.url)
    #__str__()

    @property
    def name(self):
        logger.debug("name: %s", self._name)

        if not self._name:
            self._name = nice_pkg_name(self.basename)

        return self._name

    @property
    def repo_dir(self):
        logger.debug("repo_dir: %s", self._repo_dir)

        if not self._repo_dir:
            bn = self.basename
            if bn:
                self._repo_dir = os.path.join(settings.upkg_destdir, bn)

        return self._repo_dir

    @property
    def url(self):
        logger.debug("url: %s", self._url)

        if not self._url:
            rd = self.repo_dir
            out = StringIO()
            cwd = os.getcwd()
            os.chdir(rd)

            p = git('ls-remote', '--get-url',
                        _out=out, _err=self._sh_stderr('red'))
            p.wait()
            os.chdir(cwd)
            self._url = out.getvalue().strip()
            logger.debug("URL? %s", self._url)
            out.close()

        return self._url

    @property
    def basename(self):
        logger.debug("basename: %s, repo_dir: %s", self._basename, self._repo_dir)

        if self._repo_dir:
            self._basename = os.path.basename(self._repo_dir)
        elif self._url:
            url = urlparse(self._url)
            self._basename = os.path.basename(url.path)
        else:
            self._basename = os.path.basename(pkg_name_to_path(self._name))

        return self._basename

    @property
    def installed(self):
        rd = self.repo_dir
        if rd:
            return os.path.exists(rd)

        return rd
    #installed()

    @classmethod
    def installed_list(cls):
        """todo: Docstring for list
        :return:
        :rtype:
        """
        logger.debug("")

        res = []
        for x in repo_dirlist():
            d = x['path']
            try:
                res.append(Repo(repo_dir=d, name=x['base']))
            except InvalidRepo:
                logger.warning("Invalid repo '%s' in the installation directory.", d)

        return res
    #installed_list()

    def pr_pass(self, fmt, *args, **kwargs):
        print(self.term.green(fmt.format(*args, **kwargs)))
    #pr_pass()

    def pr_info(self, fmt, *args, **kwargs):
        print(self.term.blue(fmt.format(*args, **kwargs)))
    #pr_info()

    def pr_fail(self, fmt, *args, **kwargs):
        print(self.term.red(fmt.format(*args, **kwargs)))
    #pr_fail()

    def pr_atten(self, fmt, *args, **kwargs):
        print(self.term.red(fmt.format(*args, **kwargs)))
    #pr_atten()

    def clone(self):
        """todo: Docstring for clone
        :return:
        :rtype:
        """
        logger.debug("")
    
        if not self.url:
            estr = "Cannot install this repos without a URL. %s" % self.info()
            logger.warning(estr)
            raise ValueError(estr)

        # check if the already exists, if so, don't install, update it?
        url = urlparse(self.url)
        url_path = url[2]
        path_end = url_path.split('/')
        path_end = path_end[len(path_end) - 1]

        if url.scheme not in self.supported_schemes:
            raise ValueError("Unsupported scheme '{}' for {}".format(url.scheme, self.url))

        assert self.repo_dir, "Invalid repo directory."

        # Clone it.
        logger.debug("cloning %s into %s .", self.url, self.repo_dir)
        self.pr_pass("\nInstalling %s ... " % self.url)

        p = git.clone('--progress', self.url, self.repo_dir,
                      _out=self._sh_stdout('blue'), _err=self._sh_stderr('blue'))
        p.wait()
    #clone()

    def install_update_deps(self):
        """todo: Docstring for install_update_deps
        :return:
        :rtype:
        """
        logger.debug("")
        self._ctx.installed(self.name)

        # are there any dependencies?
        depfile = os.path.join(self.repo_dir, '_upkg', 'depends')
        logger.debug("depfile? %s", depfile)
        if os.path.exists(depfile):
            logger.debug("Found depends file at %s", depfile)
            deps = open(depfile, 'r')
            dep = deps.readline()
            while dep:
                dep = dep.strip()
                logger.debug("depends: %s", dep)
                self._ctx.add_dep(nice_pkg_name(os.path.basename(dep)), dep)
                dep = deps.readline()
            deps.close()

        for rep in self._ctx.deps_needed:
            repo = Repo(url=rep)
            if repo.installed:
                repo.update()
            else:
                repo.install()
    #install_update_deps()

    def install(self):
        """todo: Docstring for install
        :return:
        :rtype:
        """
        logger.debug("")

        # If we're already installed, don't do anything.
        if self.installed:
            self.pr_info("pkg {} is already installed. Perhaps you want to update it?",
                    self.name)
            return

        self.clone()
        self.install_update_deps()

        logger.debug("Checking for install script")

        inst = os.path.join(self.repo_dir, '_upkg', 'install')
        if os.path.exists(inst):
            cwd = os.getcwd()
            logger.debug("chdir to %s", os.path.join(self.repo_dir, '_upkg'))
            logger.debug("install script is %s", inst)
            self.pr_info("Running install script at {}", inst)
            logger.debug("runnin script %s", inst)
            # We use subprocess instead of the sh module due to problems with
            # runing shell scripts with sh
            os.chdir(os.path.join(self.repo_dir, '_upkg'))
            subprocess.check_call(inst, shell=True)
            os.chdir(cwd)
            self.pr_pass("install script finished")
    #install()

    def remove(self):
        """todo: Docstring for remove
        :return:
        :rtype:
        """
        logger.debug("")

        rd = self.repo_dir

        logger.debug("pkg path %s", rd)
        if not rd:
            print(
                "unable to find pkg '%s'. %s" % (self.name, did_u_mean(self.name))
            )
            return

        # Does the repo have any uncommitted changes?
        # Is the repo out of sync(needs a push?)

        # Are you sure?
        resp = input(self.term.red("Are you sure you want to remove the '%s' pkg? [y|N] " %
                                   self.name))

        if resp == 'y' or resp == 'yes':
            self.pr_atten('removing {}...', self.name)
            shutil.rmtree(rd)

    #remove()

    def update(self):
        """todo: Docstring for update
        :return:
        :rtype:
        """
        logger.debug("")

        rd = self.repo_dir

        logger.debug("pkg path %s", rd)
        if not rd:
            print(
                "unable to find pkg '%s'. %s" % (self.name, did_u_mean(self.name))
            )

        cwd = os.getcwd()
        os.chdir(self.repo_dir)
        logger.debug("cwd: %s, updating %s ", cwd, self.repo_dir)
        try:
            p = git.pull('--rebase', '--progress',
                         _out=self._sh_stdout('blue'),
                         _err=self._sh_stderr('red'))
            p.wait()
        except Exception as e:
            pass
            # logger.warn(e)

        os.chdir(cwd)

        # Update or install any dependancies before running the
        # update script.
        self.install_update_deps()

        up = os.path.join(self.repo_dir, '_upkg', 'update')
        if os.path.exists(up):
            # We use subprocess instead of the sh module due to problems with
            # runing shell scripts with sh
            cwd = os.getcwd()
            os.chdir(os.path.join(self.repo_dir, '_upkg'))
            self.pr_info("Running update script for {} @ {}", self.name, up)
            subprocess.check_call(up, shell=True)
            os.chdir(cwd)
    #update()

    def push(self):
        """todo: Docstring for push
        :return:
        :rtype:
        """

        pass
    #push()

    def status(self):
        """Get status on the repo.
        :return:
        :rtype:
        """

        rd = self.repo_dir

        logger.debug("pkg path %s", rd)
        if not rd:
            print(
                "unable to find pkg '%s'. %s" % (self.name, did_u_mean(self.name))
            )

        cwd = os.getcwd()
        os.chdir(self.repo_dir)
        logger.debug("cwd: %s, getting status %s ", cwd, self.repo_dir)
        try:
            p = git.status(_out=self._sh_stdout('blue'),
                           _err=self._sh_stderr('red'))
            p.wait()
        except Exception:
            pass
            # logger.warn(e)
        os.chdir(cwd)

    #status()

#Repo



#!/usr/bin/env python
# encoding: utf-8

import logging

# Set up the logger
logger = logging.getLogger('upkg')
cmd_logger = logging.getLogger('command')
# Use a console handler, set it to debug by default
logger_ch = logging.StreamHandler()
logger.setLevel(logging.INFO)
log_formatter = logging.Formatter(
    ('%(asctime)s %(levelname)s:%(process)s'
                                   ' %(lineno)s:%(module)s:%(funcName)s()'
                                   ' %(message)s')
)
logger_ch.setFormatter(log_formatter)
logger.addHandler(logger_ch)

import sys
import os

if __name__ == '__main__':
    this_dir = os.path.realpath(os.path.dirname(__file__))
    sys.path.insert(0, os.path.abspath(os.path.join(this_dir, "../")))
#     sys.path.insert(0, os.path.abspath(this_dir))

import argparse
import cmds

parser = argparse.ArgumentParser(
    "upkg",
    description=("Package Yourself")
)

# parser.add_argument('-t', '--test',
#                     default='default_value',
#                     help=("Some nice help for this option")
#                     )

parser.add_argument('-d',
                    '--debug',
                    default=False, action='store_true',
                    help=("Enable debug output and debug run mode")
                    )

parser.add_argument('-c',
                    '--config',
                    default=None,
                    help=("Specify a config file location.")
                    )

def main():
    print(sys.path)
    # Sub parsers.
    sub_parser = parser.add_subparsers(help=("upkg commands"))
    cmds.build_cmds(sub_parser)

    args = parser.parse_args()
    if args.debug:
        logger.setLevel(logging.DEBUG)
        cmd_logger.setLevel(logging.DEBUG)

    logger.debug("args: %s", args)

    import conf
    conf.load_settings(args.config)

    logger.debug("settings: %s", conf.settings)

    if hasattr(args, 'func'):
        return args.func(args)
    parser.print_help()
# main()

if __name__ == '__main__':
    main()




class BaseCmd(object):
    """Docstring for Search """

    name = 'wonk'
    help_text = ("wonk it")
    aliases = []

    def __init__(self, sub_parser):
        """todo: to be defined

        :param sub_parser: arg description
        :type sub_parser: type description
        """
        self._sub_parser = sub_parser
        self._cmd_parser = self._sub_parser.add_parser(
            self.name,
            help=self.help_text,
            aliases=self.aliases,
        )
    #__init__()

    def build(self):
        self._cmd_parser.set_defaults(func=self.exec)
        return self._cmd_parser
    #build()

    def exec(self, args):
        print(args)
    #exec()
# BaseCmd



#!/usr/bin/env python
# encoding: utf-8

import os
from setuptools import setup, find_packages

def read(fname):
    res = ""
    with open(os.path.join(os.path.dirname(__file__), fname), 'r') as fd:
        res = fd.read()
    return res


setup(name='upkg',
      version='0.1.5',
      description="Package Yourself",
      url='https://github.com/jeffbuttars/upkg',
      long_description=read('README.md'),
      author="Jeff Buttars",
      author_email="jeff@jeffbuttars.com",
      packages=find_packages(exclude=['tests*']),
      license='GPLv2',
      # package_dir={'pkgs': 'pkgs'},
      # install_requires=[
      #     'tornado',
      # ],
      entry_points = {
          'console_scripts': [
              'upkg = upkg.upkg:main',
          ],
      },
      data_files=[
          # ('/etc/init.d', ['']),
      ],
      )



import logging
logger = logging.getLogger('upkg')

from blessings import Terminal

from cmds.base import BaseCmd
from upkg.lib import Repo


class Cmd(BaseCmd):
    """Docstring for Search """

    name = 'status'
    help_text = ("Get the status of what's installed")
    aliases = ['st', 'stat']

    def build(self):
        """todo: Docstring for build
        :return:
        :rtype:
        """

        self._cmd_parser.add_argument(
            'status',
            type=str,
            default=None,
            nargs="*",
            help=(""),
        )

        return super(Cmd, self).build()
    #build()

    def exec(self, args):
        """todo: Docstring for exec

        :param args: arg description
        :type args: type description
        :return:
        :rtype:
        """

        logger.debug("status %s", args.status)
        self.status(args.status)
    #exec()

    def status(self, repos):
        """

        :param repos: arg description
        :type repos: type description
        :return:
        :rtype:
        """
        logger.debug("repos: %s", repos)

        if repos:
            rlist = [Repo(name=x) for x in repos]
        else:
            # Update them all!
            rlist = Repo.installed_list()

        logger.debug("repo list: %s", rlist)

        t = Terminal()
        for r in rlist:
            logger.debug("calling status on: %s", r)
            rp = r.repo_dir

            rpw = len(rp)
            w = t.width
            bw = int(((w - rpw) / 2) - 1)
            nw = int(((w - len(r.name)) / 2) - 1)
            print(t.magenta("*" * w))
            print(t.magenta("{} {} {}".format(" " * nw, r.name, " " * nw)))
            if (rpw + 2) < w:
                print(t.magenta("{} {} {}".format(" " * bw, rp, " " * bw)))
            else:
                print(t.magenta(rpw))
            print(t.magenta("*" * w))

            r.status()
            print("\n")
        # end for r in rlist
    #status()
# Cmd



import logging
logger = logging.getLogger('upkg')

from upkg.cmds.base import BaseCmd
from upkg.lib import Repo


class Cmd(BaseCmd):
    """Docstring for Search """

    name = 'query'
    help_text = ("Advanced quering")
    aliases = ['q']

    def build(self):
        """todo: Docstring for build
        :return:
        :rtype:
        """

        self._cmd_parser.add_argument(
            'query',
            type=str,
            default=None,
            nargs="*",
            # nargs=1,
            help=("Get information about installed 'upkg'"),
        )

        return super(Cmd, self).build()
    #build()

    def exec(self, args):
        """todo: Docstring for exec

        :param args: arg description
        :type args: type description
        :return:
        :rtype:
        """

        logger.debug("query %s", args.query)

        pl = Repo.installed_list()
        if not pl:
            print("No packages installed, use 'upkg install'")

        idx = 1
        for p in pl:
            print("[%s] %s" % (idx, p))
            idx += 1
    #exec()
# Cmd
